Package: rYWAASB
Type: Package
Title: Simultaneous Selection by Trait and WAASB Index
Version: 0.4.1
Date: 2026-07-10.
Authors@R: 
    person(family="Arminian", given ="Ali", email="abeyran@gmail.com", role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0003-4749-6085"))
Maintainer: Ali Arminian <abeyran@gmail.com>
Description: This tool proposes a new ranking algorithm that utilizes a "Y*WAASB" biplot generated by the 'metan'. The aim of the current package is to effectively distinguish the top-ranked genotypes in MET (Multi-Environmental Trials). For a detailed explanation of the process of obtaining "WAASB", "WAASBY" indices, and a "Y*WAASB" biplot, refer to the manual included in this package as well as the study by Olivoto & Lúcio (2020) <doi:10.1111/2041-210X.13384>. In this context, "WAASB" refers to the "Weighted Average of Absolute Scores" provided by Olivoto et al. (2019) <doi:10.2134/agronj2019.03.0220>, which quantifies the stability of genotypes across different environments using linear mixed-effect models. To run the package, you need to extract the "WAASB" and "WAASBY" coefficients using the 'metan' and apply them. This tool utilizes PCA (Principal Component Analysis) and differentiates the entries which may be genotypes, hybrids, varieties, etc using "WAASB", "WAASBY", and a combination of the specified trait and WAASB index.
License: GPL-3
URL: https://github.com/abeyran/rYWAASB
BugReports: https://github.com/abeyran/rYWAASB/issues
Depends: R (>= 3.5)
Imports: ggplot2 (>= 3.3.0), graphics, factoextra, FactoMineR, mathjaxr
        (>= 2.0-0)
Suggests: car, metan, devtools, usethis, testthat, knitr, rmarkdown,
        roxygen2, spelling
VignetteBuilder: knitr
BuildManual: TRUE
RdMacros: mathjaxr
Encoding: UTF-8
Copyright: Ali Arminian
RoxygenNote: 8.0.0
Language: en-US
LazyData: true
LazyLoad: true
NeedsCompilation: no
Author: Ali Arminian [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4749-6085>)
Packaged: 2026-07-10 09:54:03 UTC; A.Armin
Repository: CRAN
Date/Publication: 2026-07-12 07:40:02 UTC
Built: R 4.6.1; ; 2026-07-12 10:27:01 UTC; unix

# Generated by roxygen2: do not edit by hand

export(PCA_biplot)
export(bar_plot1)
export(bar_plot2)
export(nbclust)
export(ranki)
import(ggplot2)
importFrom(FactoMineR,PCA)
importFrom(factoextra,fviz_eig)
importFrom(factoextra,fviz_nbclust)
importFrom(factoextra,fviz_pca_biplot)
importFrom(factoextra,get_eigenvalue)
importFrom(factoextra,get_pca_var)
importFrom(factoextra,hcut)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(stats,setNames)

## Summary

<!-- README.md is generated from README.Rmd. Please edit that file -->

# rYWAASB

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/README-",
  out.width = "100%"
)
```
**rYWAASB** is an R package estimating the rank of genotype based on a trait and *WAASB* stability index obtained following a stability analysis. **rYWAASB** is very easy to use and permits the simultaneously selection of genotypes based on a trait and WAASB index according to a **Y*WAASB** plot from 'metan' package.

For examples, see its [online vignette](https://github.com/abeyran/rYWAASB).

This package is licensed under the MIT license.

## Basic examples
## Download and Install

To download the release version of the package on CRAN, type the following at the R command line:

```r
install.packages('rYWAASB')
```

T run the package:

```r
library(rYWAASB)
data(maize)
ranki(maize)
```

## Reporting bugs and other issues

If you encounter a clear bug, please file a minimal reproducible example on 
[github](https://github.com/abeyran/rYWAASB).

^vignetted

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = '70%'
)

## ----Installing rYWAASB package-----------------------------------------------
if(!require('rYWAASB')){
    install.packages('rYWAASB') # call the package
}
library('rYWAASB')

## ----setup,warning=FALSE,message=FALSE----------------------------------------
## For graphical displays
library(metan)
library(ggplot2)
library(graphics)
library(factoextra)
library(FactoMineR)

## -----------------------------------------------------------------------------
waasb_model <- 
  waasb(data_ge,
        env = ENV,
        gen = GEN,
        rep = REP,
        resp = everything(),
        random = "gen", #Default
        verbose = TRUE) #Default

data <- waasb_model$GY$model
print(data)

## ----echo = TRUE, fig.height = 14, fig.width = 18, fig.align = "center", message=F, warning=F----
plot_scores(waasb_model, type = 3)

## ----Showing the maize dataset------------------------------------------------
data(maize)
head(maize)

## ----apply package by ranking the genotypes for maize data--------------------
data(maize)
ranki(maize) # or: ranki(maize, lowt = FALSE)

## ----apply package by ranking the genotypes for dm data-----------------------
data(dm)
ranki(dm, lowt = TRUE)

## ----echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F----
data(maize)
bar_plot1(maize) # or: bar_plot1(maize, lowt = FALSE)

## ----echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F----
data(dm)
bar_plot1(dm, lowt = TRUE)

## ----echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F----
data(maize)
bar_plot2(maize) # or: bar_plot2(maize, lowt = FALSE, verbose = FALSE)

## ----echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F----
data(maize)
PCA_biplot(maize) # or: PCA_biplot(maize, lowt = FALSE)

## ----echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F----
data(dm)
PCA_biplot(dm, lowt = TRUE)

## ----echo = TRUE, fig.height = 15, fig.width = 30, fig.align = "center", message=F, warning=F----
 data(maize)
 maize <- as.data.frame(maize)
 row.names(maize) <- maize[, 1]
 maize[, 1] = NULL
 GEN <- row.names(maize)
 maize <- scale(maize)
 nbclust(maize, verbose = FALSE)

 # Perform bootstrap or jackknife clustering by shipunov package. 
 # The examples should be run in the console manually due to 
 # problems occurs in the ORPHANED package "shipunov".
 #
 # library(shipunov) # recalling the shipunov package
 # 1- Bootstrap clustering:
 # data.jb <- Jclust(maize,
 #  method.d = "euclidean",
 #   method.c = "average", n.cl = 2,
 #    bootstrap = TRUE)
 #
 # plot.Jclust(data.jb, top=TRUE, lab.pos=1,
 #   lab.offset=1, lab.col=2, lab.font=2)
 # Fence(data.jb$hclust, GEN)
 #
 # data.jb <- Jclust(maize,
 #  method.d = "euclidean",
 #   method.c = "ward.D", n.cl = 2,
 #    bootstrap = TRUE)
 #
 # plot.Jclust(data.jb, top=TRUE, lab.pos=1,
 #  lab.offset=1, lab.col=2, lab.font=2)
 # Fence(data.jb$hclust, GEN)
 #
 #
 # if(verbose = TRUE):
 # cat("\nnumber of iterations:\n", data.jb$iter, "\n")
 #
 # For "bootstrap":
 # data.jb$mat <- as.matrix((data.jb$mat))
 # cat("\nmatrix of results:\n", data.jb$mat, "\n")
 # cat("clustering info, by eucledean distance measure:\n")
 # print(data.jb$hclust)
 # cat("groups:\n", data.jb$gr, "\n")
 # cat("\nsupport values:\n", data.jb$supp, "\n")
 # cat("\nnumber of clusters used:\n", data.jb$n.cl, "\n")
 #
 # 2- Jackknife clustering:
 # data.jb <- Bclust(maize,
 #   method.d = "euclidean", method.c = "average",
 #    bootstrap = FALSE)
 # plot(data.jb)
 #
 # data.jb <- Bclust(maize,
 #   method.d = "euclidean", method.c = "ward.D",
 #    bootstrap = FALSE)
 # plot(data.jb)
 #
 # if(verbose = TRUE):
 # For"jackknife":
 # cat("Consensus:\n", data.jb$consensus, "\n")
 # cat("Vlaues:\n", data.jb$values, "\n")

#' @name rYWAASB-package
#' @title Simultaneous Selection by Trait and WAASB Index
#' @docType package
#' @author {
#' Ali Arminian <abeyran@gmail.com>
#' }
#' @description
#' \pkg{rYWAASB} performs a new ranking algorithm based on a
#' "Y*WAASB" biplot generated by the 'metan' package which is
#' used in MET(Multi-Environmental Trials).
#' This package effectively distinguishes the top-ranked genotypes
#' based on a given trait (e.g. grain yield or any other trait
#' in agricultural experiments) and the "WAASB" index in the
#' Genotype-by-environment interaction effect studies.
#' Note: Fortunately, this package can impute missing
#' observations and computes them, eliminating any concerns
#' about their presence in the data set. A complete guide
#' may be found at: <https://github.com/abeyran/rYWAASB/issues>
NULL
#'
#'
#' Dataset1: a tibble containing GEN, Trait(here grain yield), `WAASB` and `WAASBY` indexes.
#' @name maize
#' @docType data
#' @usage data(maize)
#' @keywords datasets
#' @format A `data.frame` with 20 observations (genotypes)
#' within rows and columns including the trait (named as `Y`),
#' `WAASB` and `WAASBY` indexes values.
#' \describe{
#' \item{`GEN`}{a character vector saved as factor}
#' \item{`Y`}{a numeric vector}
#' \item{`WAASB`}{a numeric vector}
#' \item{`WAASBY`}{a numeric vector}
#' }
#' The input format of table of data(NA free), here *maize* data,
#' should be as follows:
#'
#' \tabular{rrrr}{
#' **GEN** \tab **Y** \tab **WAASB** \tab **WAASBY**\cr
#'  Dracma \tab 262.22 \tab 0.81 \tab 81.6\cr
#'  DKC6630 \tab 284.04 \tab 2.20 \tab 88.5\cr
#'  NS770 \tab 243.48 \tab 0.33 \tab 71.4\cr
#'    ...
#' }
#'
#' @examples
#' \donttest{
#' library(rYWAASB)
#' data(maize)
#' ranki(maize)
#' bar_plot1(maize)
#' bar_plot2(maize)
#' PCA_biplot(maize)
#' }
NULL

#' Dataset2: a tibble containing GEN, Trait (days to maturity), `WAASB` and `WAASBY` indexes.
#' @name dm
#' @docType data
#' @usage data(dm)
#' @keywords datasets
#' @format A `data.frame` with 13 observations (chickpea genotypes)
#' within rows and columns including the trait i.e. days to maturity (named as `Y`),
#' `WAASB` and `WAASBY` indexes values.
#' \describe{
#' \item{`GEN`}{a character vector saved as factor}
#' \item{`Y`}{a numeric vector}
#' \item{`WAASB`}{a numeric vector}
#' \item{`WAASBY`}{a numeric vector}
#' }
#'
#' @examples
#' \donttest{
#' library(rYWAASB)
#' data(dm)
#' ranki(dm)
#' bar_plot1(dm)
#' bar_plot2(dm)
#' PCA_biplot(dm)
#' }
NULL

#' Dataset3: a tibble containing ENV, GEN, REP factors and
#' GY(grain yield) and HM agronomic traits from the `metan` package.
#' @name data_ge
#' @docType data
#' @usage data(data_ge)
#' @keywords datasets
#' @format A `data.frame` with 420 rows in 5 columns.
#' \describe{
#' \item{`ENV`}{a character vector}
#' \item{`GEN`}{a character vector}
#' \item{`REP`}{a character vector}
#' \item{`GY`}{a numeric vector}
#' \item{`HM`}{a numeric vector}
#' }
#'
#' @examples
#' \donttest{
#' library(rYWAASB)
#' data(data_ge)
#' }
#' @references
#' Olivoto, T., & Lúcio, A.D.C.2020. metan: An R package for
#' multi‐environment trial analysis. Methods in Ecology and Evolution
#' , 11(6), 783-789.
NULL

#' @title The first barplot of the ranks of genotypes
#' @name bar_plot1
#' @author {
#' Ali Arminian <abeyran@gmail.com>
#' }
#' @description
#' `r badge('stable')`
#'
#' * `bar_plot1()` creates a bar plot for the new index (rYWAASB
#' for individuals) for simultaneous selection of genotypes by
#' trait and WAASB index using ggplot2.
#' @param datap The data set
#' @param lowt A parameter indicating whether lower rates of the trait
#' is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
#' lower values e.g. is preferred.
#' @import ggplot2
#' @return Returns an object of class `gg, ggmatrix`.
#' @usage bar_plot1(datap, lowt = FALSE)
#' @examples
#' # Case 1:  Higher trait values are preferred. For instance grain yield
#' # in cereals is a trait which its higher values are preferred and ranking
#' # is performed from the higher to lower values i.e. 1st, 2nd, 3rd etc
#' # in maize dataset.
#' \donttest{
#' data(maize)
#' bar_plot1(maize) # or: bar_plot1(maize, lowt = FALSE)
#' }
#' @examples
#' # Case 2:  In this case, the lower values of the given trait are preferred.
#' # For instance days to maturity (dm) and plant height are traits where their
#' # lower values are preferred.
#' \donttest{
#' data(dm)
#' bar_plot1(dm, lowt = TRUE)
#' }
#' @references
#' H. Wickham. ggplot2: Elegant Graphics for Data Analysis.
#' Springer-Verlag New York, 2016.
#' @export



bar_plot1 <- function(datap, lowt = FALSE)
{
  if (lowt) {
  datap <- ranki(datap, lowt = TRUE) } else {
  datap <- ranki(datap)
  }

  x <- datap$GEN; y= datap$rYWAASB

  p1 <- ggplot2::ggplot(data=datap, aes(x, y, fill=x)) +
    geom_bar(stat="identity", width = 0.50) +

    ## adjust bar labels (ranks)
    geom_text(aes(label = signif(y)), nudge_y = 3,
              hjust = 1.0, nudge_x = -.3, size = 4.0, angle=90) +
    labs(x= "Individuals", y="rYWAASB index") +

    theme(
      axis.text.x = element_text(face="bold",
                                 color="black", size=6, angle=90),
      axis.text.y = element_text(face="bold",
                                 color="black", size=6, angle=0),
      axis.title=element_text(size=16)) +

    theme(text = element_text(),
          axis.text.x = element_text(angle = 90, hjust = 1,
                                     vjust = 0.4, size = 12),
          axis.text.y = element_text(angle = 0.0, hjust = 1,
                                     size = 12)) +

    theme(
      legend.position = "none") +

    theme(
      panel.border = element_blank(),
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank(),
      panel.background = element_rect(fill = 'white'),
      plot.background = element_rect(fill = "#FFFFCC")

    )

  return(p1)
}

#' @name nbclust
#' @title Data read and estimate the cluster number
#' @author {
#' Ali Arminian <abeyran@gmail.com>
#' }
#' @description
#' `r badge('experimental')`
#'
#' `nbclust()` reads and prepares the data, and
#'  determine the optimum number of clusters using Average
#'  Silhouette Method by `factoextra` package.
#' The average silhouette approach assesses the quality of
#' clustering by evaluating how well each object fits within
#' its cluster. A high average silhouette width signifies
#' effective clustering. This method calculates the average
#' silhouette for different values of k, and the optimal
#' number of clusters (k) is the one that maximizes the
#' average silhouette across a range of potential k values.
#'
#' @details
#' The `silhouette` coefficient (SC) refers to a criterion
#' to decide number of clusters.It is defined as follows.
#' Though there are numerous methods determining number
#' of clusters such as `the gap statistic` etc.
#'
#' \loadmathjax
#' \mjsdeqn{SC = maxK{\bar{S}K}}.
#'
#' In other words, for each observation \mjseqn{i}, the
#' `silhouette width` \mjseqn{s(i)} is defined as follows:
#' Put \mjseqn{a(i)} = average dissimilarity between i and all
#' other points of the cluster to which i belongs (if i is the only
#' observation in its cluster, \mjseqn{s(i):= 0} without further
#' calculations). For all other clusters C, put \mjseqn{d(i, C)}
#' = average dissimilarity of i to all observations of C.
#' The smallest of these \mjseqn{d(i, C)} is
#' \mjseqn{b(i)=min\it(C) d(i, C)}, and can be seen as the
#' dissimilarity between i and its “neighbor” cluster, i.e.,
#' the nearest one to which it does not belong. Finally,
#' \mjsdeqn{s(i)=\frac{b(i)-a(i)}{max(a(i), b(i))}}
#'
#' * Note: The clustering methods can be: "average", "centroid",
#' "complete", "mcquitty", "median", "single", "ward.D", "ward.D2"
#' and, Distance methods can be as: "binary", "canberra", "euclidean",
#' "manhattan", "minkowski", "maximum", "pearson", "spearman", "kendall"
#' which may be used in `shipunov` or `factoextra` packages.
#' In this package we just applied `average=UPGMA` and `ward.D` algorithms.
#'
#' @param datap The data set
#' @param verbose If `verbose = TRUE` then some results are
#' @importFrom factoextra fviz_nbclust hcut
#'
#' @references
#' Lletı, R., Ortiz, M.C., Sarabia, L.A., Sánchez, M.S. 2004.
#' Selecting variables for k-means cluster analysis by using
#' a genetic algorithm that optimizes the silhouettes,
#' Analytica Chimica Acta, 515(1): 87-100.
#'
#' Rousseeuw, P.J. (1987) Silhouettes: A graphical aid to the
#' interpretation and validation of cluster analysis.
#' J. Comput. Appl. Math., 20, 53-65.
#'
#' https://uc-r.github.io/
#'
#' @return Returns a data frame
#' @usage nbclust(datap, verbose = FALSE)
#' @examples
#' \donttest{
#' library(factoextra)
#' data(maize)
#' maize <- as.data.frame(maize)
#' row.names(maize) <- maize[, 1]
#' maize[, 1] = NULL
#' GEN <- row.names(maize)
#' maize <- scale(maize)
#' nbclust(maize, verbose = FALSE)
#'
#' # Performing bootstrap or jackknife clustering
#' # by shipunov package. The examples should be run in the
#' # console manually due to problems occurs in the ORPHANED
#' # package `shipunov`.
#' #
#' # library(shipunov)
#' # 1- Bootstrap clustering:
#' # data.jb <- Jclust(maize,
#' #   method.d = "euclidean",
#' #    method.c = "average", n.cl = 2,
#' #     bootstrap = TRUE)
#' #
#' # plot.Jclust(data.jb, top=TRUE, lab.pos=1,
#' #  lab.offset=1, lab.col=2, lab.font=2)
#' # Fence(data.jb$hclust, GEN)
#' #
#' # data.jb <- Jclust(maize,
#' #  method.d = "euclidean",
#' #    method.c = "ward.D", n.cl = 2,
#' #     bootstrap = TRUE)
#' #
#' # plot.Jclust(data.jb, top=TRUE, lab.pos=1,
#' #            lab.offset=1, lab.col=2, lab.font=2)
#' # Fence(data.jb$hclust, GEN)
#' #
#' # if(verbose = TRUE):
#' # cat("\nnumber of iterations:\n", data.jb$iter, "\n")
#' #
#' # for "bootstrap":
#' # data.jb$mat <- as.matrix((data.jb$mat))
#' # data.jb$mat
#' # cat("\nmatrix of results:\n", data.jb$mat, "\n")
#' # cat("clustering info, by eucledean distance measure:\n")
#' # print(data.jb$hclust)
#' # cat("groups:\n", data.jb$gr, "\n")
#' # cat("\nsupport values:\n", data.jb$supp, "\n")
#' # cat("\nnumber of clusters used:\n", data.jb$n.cl, "\n")
#'
#' # 2- Jackknife clustering:
#' # data.jb <- Bclust(maize,
#' #   method.d = "euclidean", method.c = "average",
#' #    bootstrap = FALSE)
#' # plot(data.jb)
#' #
#' # data.jb <- Bclust(maize,
#' #   method.d = "euclidean", method.c = "ward.D",
#' #    bootstrap = FALSE)
#' # plot(data.jb)
#' #
#' # if(verbose = TRUE):
#' # For"jackknife":
#' # cat("Consensus:\n", data.jb$consensus, "\n")
#' # cat("Vlaues:\n", data.jb$values, "\n")
#' }
#' @export

nbclust <- function(datap, verbose = FALSE)
{
  datap <- data.frame(datap)

  for (i in seq_along(datap))
  {
    if(is.numeric(datap[, i])) {
      datap[, i][is.na(datap[, i])] <- mean(datap[, i], na.rm = TRUE)
      datap[, i] <- as.numeric(unlist(datap[, i]))
    }
  }

  a <- factoextra::fviz_nbclust(datap, FUNcluster = hcut,
    method = "silhouette", nboot=1000)
  n.cl = which(a$data$y == max(a$data$y))
  print(a)

  class(n.cl) <- "data frame"

  return(n.cl)

  if(verbose) {
    cat("The number of clusters using silhouette algorithm:", n.cl, "\n")
  }
}

#' @name ranki
#' @title The values and ranks of genotypes
#' @author {
#' Ali Arminian <abeyran@gmail.com>
#' }
#' @description
#' `r badge('stable')`
#'
#' `ranki()` function ranks the genotypes (or entries) based on
#' a new index utilizing the given trait and "WAASB" index to
#' simultaneous select the top-ranked ones. This can be compared
#' with WAASBY index of Olivoto (2019). We suggest users handle
#' the missing data in inputs before considering analyses,
#' due rank codes dose not implement a widespread algorithm
#' to do this task.
#' WAASB(Weighted Average of Absolute Scores), Computes
#' the Weighted Average of Absolute Scores (Olivoto et al.,
#' 2019) for quantifying the stability of *g* genotypes
#' conducted in *e* environments using linear mixed-effect models.
#'
#' @details
#' According to Olivoto et al. (2019a), WAASB(The weighted average of
#' absolute scores) is computed considering all Interaction Principal
#' Component Axis (IPCA) from the Singular Value Decomposition (SVD)
#' of the matrix of genotype-environment interaction (GEI) effects
#' generated by a linear mixed-effect model, as follows:
#'
#' \loadmathjax
#' \mjsdeqn{
#' WAASB_i = \sum_{k = 1}^{p} |IPCA_{ik} \times EP_k|/
#' \sum_{k = 1}^{p}EP_k}
#'
#' where \mjseqn{WAASB_i} is the weighted average of absolute scores
#' of the *i*th genotype; \mjseqn{IPCA_{ik}} is the score of the *i*th
#' genotype in the *k*th Interaction Principal Component Axis (IPCA);
#' and \mjseqn{EP_k} is the explained variance of the *k*th IPCA for
#' *k = 1,2,..,p*, considering \mjseqn{p=min(g-1; e-1)}.
#'
#' Further, \mjseqn{WAASBY_i} is a superiority or simultaneous
#' selection index allowing weighting between mean performance
#' and stability
#' \mjsdeqn{
#' WAASBY_i=\frac{\left({rY}_i\times\theta_Y\right)+
#' \left({rW}_i\times\theta_s\right)}{\theta_Y+\theta_s}
#' }
#' , where \mjseqn{WAASBY_i} is the superiority index for genotype
#' \mjseqn{\it{i}} that weights between mean performance and stability;
#' \mjseqn{\theta_Y} and \mjseqn{\theta_s} are the weights for
#' mean performance and stability, respectively; \mjseqn{{rY_i}} and
#' \mjseqn{{rW}_i} are the rescaled values for mean performance
#' \mjseqn{\bar{Y_i}} and stability \mjseqn{W_i}, respectively of
#' the genotype *i*. For the details of calculations, rescaling
#' and mathematics notations see (Olivoto et al., 2019).
#'
#' Finally, \mjseqn{rYWAASB_i} index is the sum of the ranks
#' (or in fact the rank of sum of ranks of  the trait and
#' WAASB index) as follows:
#' (\mjseqn{rY_i}) and WAASB index (\mjseqn{rWAASB_i}) for each
#' individual:
#'
#' \mjsdeqn{
#' rYWAASB_i = {rY_i} + {rWAASB_i}} or: =
#' \mjsdeqn{rank{{rY_i} + {rWAASB_i}}}.
#'
#' The input format of table of data(NA free), here *maize* data,
#' should be as follows:
#'
#' \tabular{rrrr}{
#' **GEN** \tab **Y** \tab **WAASB** \tab **WAASBY**\cr
#'  Dracma \tab 262.22 \tab 0.81 \tab 81.6\cr
#'  DKC6630 \tab 284.04 \tab 2.20 \tab 88.5\cr
#'  NS770 \tab 243.48 \tab 0.33 \tab 71.4\cr
#'    ...
#' }
#'
#' @param datap The data set
#' @param lowt A parameter indicating whether lower rates of the trait
#' is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
#' lower values e.g. is preferred.
#' @references
#' Olivoto, T., Lúcio, A., DC, da Silva, J.A.G., Sari, B.G.
#' and Diel, M. 2019. Mean performance and stability in
#' multi-environment trials II: Selection based on multiple
#' traits. Agronomy Journal, 111(6):2961-2969.
#'
#' Olivoto, T., & Lúcio, A.D.C.2020. metan: An R package for
#' multi‐environment trial analysis. Methods in Ecology and
#' Evolution, 11(6), 783-789.
#'
#' Kang, M.S. 1988. “A Rank-Sum Method for Selecting High-Yielding,
#' Stable Corn Genotypes.” Cereal Research Communications 16: 113–15.
#' @return Returns a data frame showing numerical rankings
#' @usage ranki(datap,  lowt = FALSE)
#' @examples
#' # Case 1:  Higher trait values are preferred. For instance grain yield
#' # in cereals is a trait which its higher values are preferred and ranking
#' # is performed from the higher to lower values i.e. 1st, 2nd, 3rd etc
#' # in maize dataset.
#' \donttest{
#' data(maize)
#' ranki(maize) # or: ranki(maize, lowt = FALSE)
#' }
#' @examples
#' # Case 2:  In this case, the lower values of the given trait are preferred.
#' # For instance days to maturity (dm) and plant height are traits where their
#' # lower values are preferred.
#' \donttest{
#' data(dm)
#' ranki(dm, lowt = TRUE)
#' }
#' @export

ranki <- function(datap, lowt = FALSE)
{
  datap <- data.frame(datap)

  for (i in seq_along(datap))
  {
    if(is.numeric(datap[, i])) {
      datap[, i][is.na(datap[, i])] <- mean(datap[, i], na.rm = TRUE)
      datap[, i] <- as.numeric(unlist(datap[, i]))
    }
  }

  n = length(datap)
  datap$GEN <- factor(datap$GEN, levels = datap$GEN)
  if (lowt) {
    datap$rY <- rank(datap$Y, na.last = NA, ties.method = "average")
    datap$rWAASBY <- rank(datap$WAASBY, na.last = NA, ties.method = "average")
  } else {
    datap$rY <- rank(-datap$Y, na.last = NA, ties.method = "average")
    datap$rWAASBY <- rank(-datap$WAASBY, na.last = NA, ties.method = "average")
  }
  datap$rWAASB <- rank(datap$WAASB, na.last = NA, ties.method = "average")
  datap$"rY+rWAASB" <- datap$rY +  datap$rWAASB
  datap$rYWAASB <- rank(datap$"rY+rWAASB", na.last = NA, ties.method = "average")

  colnames(datap) <- c("GEN", "Y=Trait", "WAASB", "WAASBY", "rY", "rWAASBY", "rWAASB", "rY+rWAASB", "rYWAASB")

  class(ranki) <- "data.frame"

  return(datap)
}

#' @title The PCA biplot with loadings
#' @name PCA_biplot
#' @author {
#' Ali Arminian <abeyran@gmail.com>
#' }
#' @description
#' `r badge('stable')`
#'
#' * `PCA_biplot()` creates the PCA (Principal Component
#'  Analysis) biplot with loadings for the new index `rYWAASB`
#'  for simultaneous selection of genotypes by trait and WAASB index.
#'  It shows `rYWAASB`, `rWAASB` and `rWAASBY` indices (r: ranked) in a
#'  biplot, simultaneously for a better differentiation of genotypes.
#'  In PCA biplots controlling the color of variable using their
#'  contrib i.e. contributions and cos2 takes place.
#'
#' @details
#' *PCA* is a machine learning method and dimension
#' reduction technique.
#' It is utilized to simplify large data sets by extracting
#' a smaller set that preserves significant patterns and
#' trends(1).
#' According to Johnson and Wichern (2007), a PCA explains
#' the var-covar structure of a set of variables
#'
#' \loadmathjax
#' \mjseqn{X_1, X_2, ..., X_p} with a less `linear`
#' combinations of such variables. Moreover the common
#' objective of PCA is 1) data reduction and 2) interpretation.
#'
#' *Biplot and PCA*:
#' The biplot is a method used to visually represent
#' both the rows and columns of a data table. It involves
#' approximating the table using a two-dimensional matrix
#' product, with the aim of creating a plane that represents
#' the rows and columns.
#' The techniques used in a biplot typically involve an eigen
#' decomposition, similar to the one used in PCA. It is common
#' for the biplot to be conducted using mean-centered and scaled
#' data(2). For scaling variables, the data can be transformed as follow:
#' \mjsdeqn{z = \frac{x-\bar{x}}{s(x)}}
#' where 's(x)' denotes the sample standard deviation of 'x' parameter, calculated as:
#' \mjsdeqn{s = \sqrt{\frac{1}{n-1}\sum_{i=1}^{n}(x_i-\bar{x})^2}}
#' *Algebra of PCA:*
#' As Johnson and Wichern (2007) stated(3), if the random vector
#' \mjseqn{\mathbf{X'} = \{X_1, X_2,...,X_p \}} have the
#' covariance matrix \mjseqn{\sum} with eigenvalues
#' \mjseqn{\lambda_1 \ge \lambda_2 \ge ... \ge \lambda_p \ge 0}.
#'
#' Regarding the linear combinations:
#' \mjsdeqn{Y_1 = a'_1X = a_{11}X_1 + a_{12}X_2 + ... + a_{1P}X_p }
#' \mjsdeqn{Y_2 = a'_2X = a_{21}X_1 + a_{22}X_2 + ... + a_{2p}X_p}
#' \mjsdeqn{...}
#' \mjsdeqn{Y_p = a'_pX = a_{p1}X_1 + a_{p2}X_2 + ... + a_{pp}X_p}
#'
#' where \mjseqn{Var(Y_i) = \mathbf{a'_i\sum{a_i}}} ,
#' i = 1, 2, ..., p
#' \mjseqn{Cov(Y_i, Y_k) = \mathbf{a'_i\sum{a_k}}} ,
#' i, k = 1, 2, ..., p
#'
#' The principal components refer to the uncorrelated
#' linear combinations
#' \mjseqn{Y_1, Y_2, ..., Y_p} which aim to have the
#' largest possible variances.
#'
#' For the random vector \mjseqn{\mathbf{X'}=\left [ X_1, X_2, ...,
#' X_p \right ]},
#' if \mjseqn{\mathbf{\sum}} be the associated covariance matrix, then
#' \mjseqn{\mathbf{\sum}} have the eigenvalue-eigenvector pairs
#' \mjseqn{(\lambda_1, e_1), (\lambda_2, e_2), ..., (\lambda_p, e_p)},
#' and as said \mjseqn{\lambda_1 \ge \lambda_2 \ge ... \ge \lambda_p \ge 0}.
#'
#' Then the \mjseqn{\it{i}}th principal component is as follows:
#' \mjsdeqn{Y_i = \mathbf{e'_iX} = e_{i1}X_1 + e_{i2}X_2 + ... + e_{ip}X_p,
#' i = 1, 2, ..., p}, where \mjseqn{Var(Y_i) =\mathbf(e'_i\sum{e_i}) = \lambda_i,
#' i = 1, 2, ..., p}
#' \mjseqn{Cov(Y_i, Y_k) = \mathbf{e'_i\sum e_i = 0, i \not\equiv k}}, and:
#' \mjseqn{\sigma_{11} + \sigma_{22} + ... + \sigma_{pp} =
#' \sum_{i=1}^p{Var(X_i)} = \lambda_1 +
#' \lambda_2 + ... + \lambda_p = \sum_{i=1}^p{Var(Y_i)}}.
#'
#' Interestingly, Total population variance = \mjseqn{\sigma_{11} + \sigma_{22}
#' + ... + \sigma_{pp} = \lambda_1 + \lambda_2 + ... + \lambda_{p}}.
#'
#' Another issues that are significant in PCA analysis are:
#' 1) The proportion of total variance due to (explained by)
#' the \mjseqn{\mathit{k}}th principal component:
#' \mjsdeqn{\frac{\lambda_k}{(\lambda_1 + \lambda_2 + ... +
#' \lambda_p)},  k=1, 2, ..., p}
#'
#' 2) The correlation coefficients between the components \mjseqn{Y_i}
#' and the variables \mjseqn{X_k} is as follows:
#' \mjseqn{\rho_{Y_i, X_k} = \frac{e_{ik}\sqrt{\lambda_i}}{\sqrt{\sigma_{kk}}}},
#' i,k = 1, 2, ..., p
#'
#' Please note that PCA can be performed on `Covariance` or
#' `correlation matrices`.
#' And before PCA the data should be centered, generally.
#'
#' @param datap The data set
#' @param lowt A parameter indicating whether lower rates of the trait
#' is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
#' lower values e.g. is preferred.
#' @importFrom FactoMineR PCA
#' @importFrom graphics par
#' @importFrom stats setNames
#' @importFrom factoextra get_eigenvalue fviz_eig fviz_pca_biplot get_pca_var
#' @return Returns a a list of dataframes
#' @usage PCA_biplot(datap, lowt = FALSE)
#'
#' @examples
#' # Case 1: for maize dataset, grain yield
#' \donttest{
#' data(maize)
#' PCA_biplot(maize) # or: PCA_biplot(maize, lowt = FALSE)
#' }
#' @examples
#' # Case 2: for days to maturity (dm) trait of chickpea
#' \donttest{
#' data(dm)
#' PCA_biplot(dm, lowt = TRUE)
#' }
#' @references
#' (1) <https://builtin.com>
#'
#' (2) <https://pca4ds.github.io/biplot-and-pca.html>.
#'
#' (3) Johnson, R.A. and Wichern, D.W. 2007. Applied
#' Multivariate Statistical Analysis. Pearson Prentice
#' Hall. 773 p.
#'
#' @export

PCA_biplot <- function(datap, lowt = FALSE)
{
  if (lowt) {
    datap <- ranki(datap, lowt = TRUE) } else {
      datap <- ranki(datap)
    }

  row.names(datap) <- datap$GEN
  datap[, 1] <- NULL
  datap <- datap[, c(1, 3, 5, 6, 8)]

  res.pca <- FactoMineR::PCA(datap, graph = FALSE)

  var <- get_pca_var(res.pca)

  eignv <- factoextra::get_eigenvalue(res.pca)

  varout <- list(var$coord, var$cos2, var$contrib, eignv)
  class(varout) <- "list"

  varout <- setNames(varout, c("Coordinates of variables making a scatter plot", "Squaring the variable coordinates: var.cos2 = var.coord^2", "The % contributions of the variables to the PCs", "eigenvalues data"))
  print(varout)


  old.par <- par(no.readonly = TRUE)
  par(mar=rep(1, 4))
  on.exit(par(old.par))

  # scree plot
  p1 <- factoextra::fviz_eig(res.pca, addlabels = TRUE)
  print(p1)


  # Biplot of individuals and variables
  p1 <- factoextra::fviz_pca_biplot(res.pca, axes = c(1, 2), geom = c("point", "text"), geom.ind = c("point", "text"), geom.var = c("arrow", "text"), col.ind = "black", fill.ind = "white", col.var = "red", fill.var = "white", gradient.cols = NULL, label = "all", invisible = "none", repel = TRUE, habillage = "none", palette = NULL, addEllipses = FALSE)
  print(p1)


  # Control variable colors using their contributions
  p1 <- factoextra::fviz_pca_var(res.pca, axes = c(1, 2), geom = c("point", "text"), geom.ind = c("point", "text"), geom.var = c("arrow", "text"), col.ind = "black", fill.ind = "white", col.var = "contrib", fill.var = "white", gradient.cols = c("white", "blue", "red"), ggtheme = theme_minimal())
  print(p1)


  # Contributions of variables to PC1
  p1 <- factoextra::fviz_contrib(res.pca, choice = "var", axes = 1)
  print(p1)

  # Contributions of variables to PC2
  p1 <- factoextra::fviz_contrib(res.pca, choice = "var", axes = 2)
  print(p1)


  # Graph of individuals
  p1 <- factoextra::fviz_pca_ind(res.pca, col.ind="cos2", geom = c("point", "text"), gradient.cols = c("white", "#2E9FDF", "#FC4E07" ))
  print(p1)

}

#' @title The second barplot of the ranks of genotypes
#' @name bar_plot2
#' @author {
#' Ali Arminian <abeyran@gmail.com>
#' }
#' @description
#' `r badge('stable')`
#'
#' * `bar_plot2()` creates the 2nd barplot of the ranks of genotypes
#' using the `graphics` package.
#' @param datap The data set
#' @param lowt A parameter indicating whether lower rates of the trait
#' is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
#' lower values e.g. is preferred.
#' @param verbose If `verbose = TRUE` then some results are printed
#' @importFrom graphics barplot par text
#' @return Returns an object of class `gg, graphics`
#' @usage bar_plot2(datap, lowt = FALSE, verbose = FALSE)
#'
#' @examples
#' # Case 1:  Higher trait values are preferred. For instance grain yield
#' # in cereals is a trait which its higher values are preferred and ranking
#' # is performed from the higher to lower values i.e. 1st, 2nd, 3rd etc
#' # in maize dataset.
#' \donttest{
#' data(maize)
#' bar_plot2(maize) # or: bar_plot2(maize, lowt = FALSE, verbose = FALSE)
#' }
#' @examples
#' # Case 2:  In this case, the lower values of the given trait are preferred.
#' # For instance days to maturity (dm) and plant height are traits where their
#' # lower values are preferred.
#' \donttest{
#' data(dm)
#' bar_plot2(dm, lowt = TRUE, verbose = TRUE)
#' }
#' @export

bar_plot2 <- function(datap, lowt = FALSE, verbose = FALSE)
{
  if (lowt) {
    datap <- ranki(datap, lowt = TRUE) } else {
      datap <- ranki(datap) }

  A <- datap$rYWAASB
  B <- datap$GEN

  if (verbose) {
    cat("Plot the bar chart with text features:", "\n")
  }

  op <- par(no.readonly=TRUE)
  par(mar=c(6, 4, 2, 0.5))

  p1 <- barplot(A, names.arg = B,
          xlab = NULL, space=0.75,
          ylab = "rYWAASB index", col = "steelblue",
          ylim = c(0, (max(A) + 1)),
          main = "Ranking by trait and WAASB index",
          cex.main = 1.0, cex.lab = 1.2, #cex.lab: axis titles font
          cex.axis = 1.0, las=2) # cex.axis: y.axis scale font

          text(x = p1, y = A+0.5, labels = A)
  return(p1)
          par(op)

}

---
title: "Simultaneous Selection by Trait and WAASB Index"
author: "Ali Arminian"
date: "`r Sys.Date()`"
always_allow_html: yes
output: rmarkdown::html_vignette
fig_caption: yes
link-citations: true
toc: true
bibliography: rywaasbref.bib
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{rYWAASB-vignette}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = '70%'
)
```


## 1-Introduction
`I dedicate this package to my colleague, Professor Dr. Manjit S. Kang, a great biostatistician and quantitative geneticist, who passed away recently and has devoted his life to PLANT BREEDING science and has provided me with invaluable supports!.`

 **Stability analysis** is crucial in plant breeding to select superior genotypes that perform consistently across different environments. Several models and methods have been developed for this purpose, including Additive Main Effect and Multiplicative Interaction (*AMMI*), Weighted Average of Absolute Scores (*WAASB*), and Genotype plus Genotype-Environment (*GGE*) interactions in multi-environmental trials (*MET*) [@mishra2024; @sakata2021; @pour2022; @danakumara2023]. These analyses help breeders identify genotypes with stable performance and adaptability under varying conditions, leading to optimal yield stability and successful crop production. Utilizing these stability analysis tools, breeders can navigate the complexities of genotype-environment interactions and select genotypes that consistently excel across different locations and seasons, ensuring the development of resilient and high-performing plant varieties [@swarup2014].

## The WAASB and WAASBY objects
By combining the strengths of AMMI for assessing stability and BLUP for prediction accuracy, breeders can effectively select genotypes that consistently perform well across different environmental conditions. This is crucial for developing sustainable agricultural systems and improving food security. To estimate the stability index of genotypes in multi-environment trials (METs), the **WAASB** index (weighted average of the absolute values obtained from the singular value decomposition of the BLUP matrix for the genotype by environment interaction effects, generated by a linear mixed-effects model) is calculated using the formula provided by [@olivoto2019]. They suggest that the function `*waasb()*` computes the Weighted Average of the Absolute Scores considering all possible IPCA from the Singular Value Decomposition of the BLUPs for genotype-vs-environment interaction effects obtained by an Linear Mixed-effect Model [@olivoto2019]
, as follows:

$$ WAASB_i  = \sum_{k = 1}^{p} |IPCA_{ik} \times EP_k|/ \sum_{k = 1}^{p}EP_k $$

where $WAASB_i$ is the weighted average of absolute scores of the *i*th genotype; $IPCA_{ik}$ is the scores of the *i*th genotype in the *k*th IPCA; and $EP_k$ is the explained variance of the *k*th PCA for $k = 1,2,..,p$, $p = min(g-1; e-1)$.

Further, **WAABSY** is a superiority or simultaneous selection index allowing weighting between mean performance and stability [@olivoto2019]: 

$$ WAASBY_i=\frac{\left({rY}_i\times\theta_Y\right)+\left({rW}_i\times\theta_s\right)}{\theta_Y+\theta_s} $$
where WAASBYi is the superiority index for genotype \it{i} that weights between mean performance and stability; $\theta_Y$ and $\theta_s$ are the weights for mean performance and stability, respectively; \{rY}_i and \{rW}_i are the rescaled values for mean performance $\bar{Y_i}$ and stability $W_i$, respectively of the genotype i. For the details of calculations, rescalling and mathematics notations see [@olivoto2019].

In this package, my new index (*rYWAASB*) gives results which can be compared with *WAASBY* index provided by Olivoto et al. [@olivoto2019].

For working with *rYWAASB* package, firstly, if the *metan* or *rYWAASB* packages are not already installed, they should be installed on the system. The analysis requires the following packages to be installed: 

```{r Installing rYWAASB package}
if(!require('rYWAASB')){
    install.packages('rYWAASB') # call the package
}
library('rYWAASB')
```

The analyses requires the following R packages:
```{r setup,warning=FALSE,message=FALSE}
## For graphical displays
library(metan)
library(ggplot2)
library(graphics)
library(factoextra)
library(FactoMineR)
```

The codes provided below form the *metan* package, allow you to access the *WAASB* index values, rankings, and other information for genotypes (or entries) in general.


```{r  }
waasb_model <- 
  waasb(data_ge,
        env = ENV,
        gen = GEN,
        rep = REP,
        resp = everything(),
        random = "gen", #Default
        verbose = TRUE) #Default

data <- waasb_model$GY$model
print(data)
```

The output generated by the *waasb()* function is very similar to that generated by the *waas()* function. The main difference is that the singular value decomposition is based on the BLUP for GEI effects matrix. For more information, refer to [@olivoto2019].


Several indexes have been developed to identify genotypes that exhibit both high performance and stability in plant breeding programs. One example is the *kangranksum* index, which was developed by Kang [@kang1988]. This index combines yield and stability ranks based on the *Shukla* stability index. Olivoto [@olivoto2019], on the other hand, created the *WAASBY* index by assigning weights to yield and stability. Our rYWAASB index can be compared to these earlier indexes, as it follows a similar computational approach. However, our index (*rYWAASB*) is a powerful tool, because it incorporates a trait and *WAASB* rankings in the process.


## 2- How to apply the package
First, let's examine the `Y*WAASB` biplot generated by the *metan* package. This will allow us to compare the results of the *rYWAASB* package with the `Y*WAASB` biplot. In the `Y*WAASB` or `GY*WAASB` biplot proposed by [@olivoto2019] (Fig. 1), the quadrants illustrate the stability and trait patterns (specifically, the grain yield of oat genotypes in the data_ge dataset) as follows:

- Quadrant I: This quadrant consists of unstable genotypes or environments that have high discrimination ability but productivity below the grand mean.
- Quadrant II: This quadrant includes unstable genotypes with productivity above the grand mean.
- Quadrant III: This quadrant comprises low productive but stable genotypes.
- Quadrant IV: This quadrant contains productive and broadly adapted genotypes.


```{r  echo = TRUE, fig.height = 14, fig.width = 18, fig.align = "center", message=F, warning=F}
plot_scores(waasb_model, type = 3)
```

Fig. 1: Y*WAASB biplot of `metan` package built-in oat data


In this section, we will utilize the built-in data *maize* to generate ranking scores for different genotypes, along with their corresponding plots. For further details, please refer to the *?maize* documentation. It is also possible to use other datasets as long as they contain the following columns: genotype, trait, and WAASB index for genotypes. To understand how the HTML tables were created, please refer to the [Rendering engine](#rendering) section.


```{r Showing the maize dataset}
data(maize)
head(maize)
```

## Applying the **rYWAASB** package. Ranking table:
We recommend that users address (handle/overcome/substitute) any missing data in their inputs before proceeding with analyses. This is because the rank codes do not incorporate a comprehensive algorithm to handle this task.

Case 1. Higher trait values are preferred. For instance grain yield in cereals is a trait which its higher values are preferred and ranking is performed from the higher to lower values i.e. 1st, 2nd, 3rd etc in maize dataset.

```{r apply package by ranking the genotypes for maize data}
data(maize)
ranki(maize) # or: ranki(maize, lowt = FALSE)
```
In the table above, the genotype with the lowest rank (*Dracma*) is considered the best due to its high grain yield and low *WAASB* score.

Case 2. 
In this case, the lower values of the given trait are preferred. For instance days to maturity and plant height are traits where their lower values are preferred.

```{r apply package by ranking the genotypes for dm data}
data(dm)
ranki(dm, lowt = TRUE)
```

## Drawing the first plot *bar_plot1*:
case 1. *maize* dataset:
```{r  echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F}
data(maize)
bar_plot1(maize) # or: bar_plot1(maize, lowt = FALSE)
```

case 2. *dm* dataset:
```{r  echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F}
data(dm)
bar_plot1(dm, lowt = TRUE)
```


Fig.2. The first barplot of the `rYWASSB` package


## Drawing the second plot *bar_plot2*:
```{r  echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F}
data(maize)
bar_plot2(maize) # or: bar_plot2(maize, lowt = FALSE, verbose = FALSE)
```

Fig.3. The second barplot of the `rYWASSB` package

## Performing PCA and textual and graphical representations (scree plot and PCA-dependent biplots (3 plots))
case 1. For grain yield data:

```{r  echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F}
data(maize)
PCA_biplot(maize) # or: PCA_biplot(maize, lowt = FALSE)
```

Figs.4-6. The PCA biplot with loadings for compare genotypes

Case 2. For days to maturity data:
```{r  echo = TRUE, fig.height = 14, fig.width = 20, fig.align = "center", message=F, warning=F}
data(dm)
PCA_biplot(dm, lowt = TRUE)
```

Figs.7-9. The PCA biplot with loadings for compare genotypes

## Designing the cluster analysis on data and define the optimum number of clusters uning `The Average Silhouette Method`. The details of this method is given in the *description* and *details* parts of the function `nbclust()`. For running the cluster analysis by menas of `shipunov` package (courtesy of late professor Alexy Shipunov) act as:



```{r  echo = TRUE, fig.height = 15, fig.width = 30, fig.align = "center", message=F, warning=F}
 data(maize)
 maize <- as.data.frame(maize)
 row.names(maize) <- maize[, 1]
 maize[, 1] = NULL
 GEN <- row.names(maize)
 maize <- scale(maize)
 nbclust(maize, verbose = FALSE)

 # Perform bootstrap or jackknife clustering by shipunov package. 
 # The examples should be run in the console manually due to 
 # problems occurs in the ORPHANED package "shipunov".
 #
 # library(shipunov) # recalling the shipunov package
 # 1- Bootstrap clustering:
 # data.jb <- Jclust(maize,
 #  method.d = "euclidean",
 #   method.c = "average", n.cl = 2,
 #    bootstrap = TRUE)
 #
 # plot.Jclust(data.jb, top=TRUE, lab.pos=1,
 #   lab.offset=1, lab.col=2, lab.font=2)
 # Fence(data.jb$hclust, GEN)
 #
 # data.jb <- Jclust(maize,
 #  method.d = "euclidean",
 #   method.c = "ward.D", n.cl = 2,
 #    bootstrap = TRUE)
 #
 # plot.Jclust(data.jb, top=TRUE, lab.pos=1,
 #  lab.offset=1, lab.col=2, lab.font=2)
 # Fence(data.jb$hclust, GEN)
 #
 #
 # if(verbose = TRUE):
 # cat("\nnumber of iterations:\n", data.jb$iter, "\n")
 #
 # For "bootstrap":
 # data.jb$mat <- as.matrix((data.jb$mat))
 # cat("\nmatrix of results:\n", data.jb$mat, "\n")
 # cat("clustering info, by eucledean distance measure:\n")
 # print(data.jb$hclust)
 # cat("groups:\n", data.jb$gr, "\n")
 # cat("\nsupport values:\n", data.jb$supp, "\n")
 # cat("\nnumber of clusters used:\n", data.jb$n.cl, "\n")
 #
 # 2- Jackknife clustering:
 # data.jb <- Bclust(maize,
 #   method.d = "euclidean", method.c = "average",
 #    bootstrap = FALSE)
 # plot(data.jb)
 #
 # data.jb <- Bclust(maize,
 #   method.d = "euclidean", method.c = "ward.D",
 #    bootstrap = FALSE)
 # plot(data.jb)
 #
 # if(verbose = TRUE):
 # For"jackknife":
 # cat("Consensus:\n", data.jb$consensus, "\n")
 # cat("Vlaues:\n", data.jb$values, "\n")
```


Figures 10-12 Cluster analysis by 1000 bootstrap iterations and determine optimum N clusters using The Average Silhouette algorithm.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PCA_biplot.R
\name{PCA_biplot}
\alias{PCA_biplot}
\title{The PCA biplot with loadings}
\usage{
PCA_biplot(datap, lowt = FALSE)
}
\arguments{
\item{datap}{The data set}

\item{lowt}{A parameter indicating whether lower rates of the trait
is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
lower values e.g. is preferred.}
}
\value{
Returns a a list of dataframes
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}
\itemize{
\item \code{PCA_biplot()} creates the PCA (Principal Component
Analysis) biplot with loadings for the new index \code{rYWAASB}
for simultaneous selection of genotypes by trait and WAASB index.
It shows \code{rYWAASB}, \code{rWAASB} and \code{rWAASBY} indices (r: ranked) in a
biplot, simultaneously for a better differentiation of genotypes.
In PCA biplots controlling the color of variable using their
contrib i.e. contributions and cos2 takes place.
}
}
\details{
\emph{PCA} is a machine learning method and dimension
reduction technique.
It is utilized to simplify large data sets by extracting
a smaller set that preserves significant patterns and
trends(1).
According to Johnson and Wichern (2007), a PCA explains
the var-covar structure of a set of variables

\loadmathjax
\mjseqn{X_1, X_2, ..., X_p} with a less \code{linear}
combinations of such variables. Moreover the common
objective of PCA is 1) data reduction and 2) interpretation.

\emph{Biplot and PCA}:
The biplot is a method used to visually represent
both the rows and columns of a data table. It involves
approximating the table using a two-dimensional matrix
product, with the aim of creating a plane that represents
the rows and columns.
The techniques used in a biplot typically involve an eigen
decomposition, similar to the one used in PCA. It is common
for the biplot to be conducted using mean-centered and scaled
data(2). For scaling variables, the data can be transformed as follow:
\mjsdeqn{z = \frac{x-\bar{x}}{s(x)}}
where 's(x)' denotes the sample standard deviation of 'x' parameter, calculated as:
\mjsdeqn{s = \sqrt{\frac{1}{n-1}\sum_{i=1}^{n}(x_i-\bar{x})^2}}
\emph{Algebra of PCA:}
As Johnson and Wichern (2007) stated(3), if the random vector
\mjseqn{\mathbf{X'} = \{X_1, X_2,...,X_p \}} have the
covariance matrix \mjseqn{\sum} with eigenvalues
\mjseqn{\lambda_1 \ge \lambda_2 \ge ... \ge \lambda_p \ge 0}.

Regarding the linear combinations:
\mjsdeqn{Y_1 = a'_1X = a_{11}X_1 + a_{12}X_2 + ... + a_{1P}X_p }
\mjsdeqn{Y_2 = a'_2X = a_{21}X_1 + a_{22}X_2 + ... + a_{2p}X_p}
\mjsdeqn{...}
\mjsdeqn{Y_p = a'_pX = a_{p1}X_1 + a_{p2}X_2 + ... + a_{pp}X_p}

where \mjseqn{Var(Y_i) = \mathbf{a'_i\sum{a_i}}} ,
i = 1, 2, ..., p
\mjseqn{Cov(Y_i, Y_k) = \mathbf{a'_i\sum{a_k}}} ,
i, k = 1, 2, ..., p

The principal components refer to the uncorrelated
linear combinations
\mjseqn{Y_1, Y_2, ..., Y_p} which aim to have the
largest possible variances.

For the random vector \mjseqn{\mathbf{X'}=\left [ X_1, X_2, ...,
X_p \right ]},
if \mjseqn{\mathbf{\sum}} be the associated covariance matrix, then
\mjseqn{\mathbf{\sum}} have the eigenvalue-eigenvector pairs
\mjseqn{(\lambda_1, e_1), (\lambda_2, e_2), ..., (\lambda_p, e_p)},
and as said \mjseqn{\lambda_1 \ge \lambda_2 \ge ... \ge \lambda_p \ge 0}.

Then the \mjseqn{\it{i}}th principal component is as follows:
\mjsdeqn{Y_i = \mathbf{e'_iX} = e_{i1}X_1 + e_{i2}X_2 + ... + e_{ip}X_p,
i = 1, 2, ..., p}, where \mjseqn{Var(Y_i) =\mathbf(e'_i\sum{e_i}) = \lambda_i,
i = 1, 2, ..., p}
\mjseqn{Cov(Y_i, Y_k) = \mathbf{e'_i\sum e_i = 0, i \not\equiv k}}, and:
\mjseqn{\sigma_{11} + \sigma_{22} + ... + \sigma_{pp} =
\sum_{i=1}^p{Var(X_i)} = \lambda_1 +
\lambda_2 + ... + \lambda_p = \sum_{i=1}^p{Var(Y_i)}}.

Interestingly, Total population variance = \mjseqn{\sigma_{11} + \sigma_{22}
+ ... + \sigma_{pp} = \lambda_1 + \lambda_2 + ... + \lambda_{p}}.

Another issues that are significant in PCA analysis are:
\enumerate{
\item The proportion of total variance due to (explained by)
the \mjseqn{\mathit{k}}th principal component:
\mjsdeqn{\frac{\lambda_k}{(\lambda_1 + \lambda_2 + ... +
\lambda_p)},  k=1, 2, ..., p}
\item The correlation coefficients between the components \mjseqn{Y_i}
and the variables \mjseqn{X_k} is as follows:
\mjseqn{\rho_{Y_i, X_k} = \frac{e_{ik}\sqrt{\lambda_i}}{\sqrt{\sigma_{kk}}}},
i,k = 1, 2, ..., p
}

Please note that PCA can be performed on \code{Covariance} or
\verb{correlation matrices}.
And before PCA the data should be centered, generally.
}
\examples{
# Case 1: for maize dataset, grain yield
\donttest{
data(maize)
PCA_biplot(maize) # or: PCA_biplot(maize, lowt = FALSE)
}
# Case 2: for days to maturity (dm) trait of chickpea
\donttest{
data(dm)
PCA_biplot(dm, lowt = TRUE)
}
}
\references{
(1) \url{https://builtin.com}

(2) \url{https://pca4ds.github.io/biplot-and-pca.html}.

(3) Johnson, R.A. and Wichern, D.W. 2007. Applied
Multivariate Statistical Analysis. Pearson Prentice
Hall. 773 p.
}
\author{
{
Ali Arminian \href{mailto:abeyran@gmail.com}{abeyran@gmail.com}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rYWAASB.R
\docType{data}
\name{maize}
\alias{maize}
\title{Dataset1: a tibble containing GEN, Trait(here grain yield), \code{WAASB} and \code{WAASBY} indexes.}
\format{
A \code{data.frame} with 20 observations (genotypes)
within rows and columns including the trait (named as \code{Y}),
\code{WAASB} and \code{WAASBY} indexes values.
\describe{
\item{\code{GEN}}{a character vector saved as factor}
\item{\code{Y}}{a numeric vector}
\item{\code{WAASB}}{a numeric vector}
\item{\code{WAASBY}}{a numeric vector}
}
The input format of table of data(NA free), here \emph{maize} data,
should be as follows:

\tabular{rrrr}{
\strong{GEN} \tab \strong{Y} \tab \strong{WAASB} \tab \strong{WAASBY}\cr
Dracma \tab 262.22 \tab 0.81 \tab 81.6\cr
DKC6630 \tab 284.04 \tab 2.20 \tab 88.5\cr
NS770 \tab 243.48 \tab 0.33 \tab 71.4\cr
...
}
}
\usage{
data(maize)
}
\description{
Dataset1: a tibble containing GEN, Trait(here grain yield), \code{WAASB} and \code{WAASBY} indexes.
}
\examples{
\donttest{
library(rYWAASB)
data(maize)
ranki(maize)
bar_plot1(maize)
bar_plot2(maize)
PCA_biplot(maize)
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bar_plot1.R
\name{bar_plot1}
\alias{bar_plot1}
\title{The first barplot of the ranks of genotypes}
\usage{
bar_plot1(datap, lowt = FALSE)
}
\arguments{
\item{datap}{The data set}

\item{lowt}{A parameter indicating whether lower rates of the trait
is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
lower values e.g. is preferred.}
}
\value{
Returns an object of class \verb{gg, ggmatrix}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}
\itemize{
\item \code{bar_plot1()} creates a bar plot for the new index (rYWAASB
for individuals) for simultaneous selection of genotypes by
trait and WAASB index using ggplot2.
}
}
\examples{
# Case 1:  Higher trait values are preferred. For instance grain yield
# in cereals is a trait which its higher values are preferred and ranking
# is performed from the higher to lower values i.e. 1st, 2nd, 3rd etc
# in maize dataset.
\donttest{
data(maize)
bar_plot1(maize) # or: bar_plot1(maize, lowt = FALSE)
}
# Case 2:  In this case, the lower values of the given trait are preferred.
# For instance days to maturity (dm) and plant height are traits where their
# lower values are preferred.
\donttest{
data(dm)
bar_plot1(dm, lowt = TRUE)
}
}
\references{
H. Wickham. ggplot2: Elegant Graphics for Data Analysis.
Springer-Verlag New York, 2016.
}
\author{
{
Ali Arminian \href{mailto:abeyran@gmail.com}{abeyran@gmail.com}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rYWAASB.R
\docType{data}
\name{dm}
\alias{dm}
\title{Dataset2: a tibble containing GEN, Trait (days to maturity), \code{WAASB} and \code{WAASBY} indexes.}
\format{
A \code{data.frame} with 13 observations (chickpea genotypes)
within rows and columns including the trait i.e. days to maturity (named as \code{Y}),
\code{WAASB} and \code{WAASBY} indexes values.
\describe{
\item{\code{GEN}}{a character vector saved as factor}
\item{\code{Y}}{a numeric vector}
\item{\code{WAASB}}{a numeric vector}
\item{\code{WAASBY}}{a numeric vector}
}
}
\usage{
data(dm)
}
\description{
Dataset2: a tibble containing GEN, Trait (days to maturity), \code{WAASB} and \code{WAASBY} indexes.
}
\examples{
\donttest{
library(rYWAASB)
data(dm)
ranki(dm)
bar_plot1(dm)
bar_plot2(dm)
PCA_biplot(dm)
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rYWAASB.R
\docType{package}
\name{rYWAASB-package}
\alias{rYWAASB}
\alias{rYWAASB-package}
\title{Simultaneous Selection by Trait and WAASB Index}
\description{
\pkg{rYWAASB} performs a new ranking algorithm based on a
"Y*WAASB" biplot generated by the 'metan' package which is
used in MET(Multi-Environmental Trials).
This package effectively distinguishes the top-ranked genotypes
based on a given trait (e.g. grain yield or any other trait
in agricultural experiments) and the "WAASB" index in the
Genotype-by-environment interaction effect studies.
Note: Fortunately, this package can impute missing
observations and computes them, eliminating any concerns
about their presence in the data set. A complete guide
may be found at: \url{https://github.com/abeyran/rYWAASB/issues}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/abeyran/rYWAASB}
  \item Report bugs at \url{https://github.com/abeyran/rYWAASB/issues}
}

}
\author{
{
Ali Arminian \href{mailto:abeyran@gmail.com}{abeyran@gmail.com}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rYWAASB.R
\docType{data}
\name{data_ge}
\alias{data_ge}
\title{Dataset3: a tibble containing ENV, GEN, REP factors and
GY(grain yield) and HM agronomic traits from the \code{metan} package.}
\format{
A \code{data.frame} with 420 rows in 5 columns.
\describe{
\item{\code{ENV}}{a character vector}
\item{\code{GEN}}{a character vector}
\item{\code{REP}}{a character vector}
\item{\code{GY}}{a numeric vector}
\item{\code{HM}}{a numeric vector}
}
}
\usage{
data(data_ge)
}
\description{
Dataset3: a tibble containing ENV, GEN, REP factors and
GY(grain yield) and HM agronomic traits from the \code{metan} package.
}
\examples{
\donttest{
library(rYWAASB)
data(data_ge)
}
}
\references{
Olivoto, T., & Lúcio, A.D.C.2020. metan: An R package for
multi‐environment trial analysis. Methods in Ecology and Evolution
, 11(6), 783-789.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ranki.R
\name{ranki}
\alias{ranki}
\title{The values and ranks of genotypes}
\usage{
ranki(datap,  lowt = FALSE)
}
\arguments{
\item{datap}{The data set}

\item{lowt}{A parameter indicating whether lower rates of the trait
is preferred or not. For grain yield e.g. Upper values is preferred. For plant height
lower values e.g. is preferred.}
}
\value{
Returns a data frame showing numerical rankings
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#stable}{\figure{lifecycle-stable.svg}{options: alt='[Stable]'}}}{\strong{[Stable]}}

\code{ranki()} function ranks the genotypes (or entries) based on
a new index utilizing the given trait and "WAASB" index to
simultaneous select the top-ranked ones. This can be compared
with WAASBY index of Olivoto (2019). We suggest users handle
the missing data in inputs before considering analyses,
due rank codes dose not implement a widespread algorithm
to do this task.
WAASB(Weighted Average of Absolute Scores), Computes
the Weighted Average of Absolute Scores (Olivoto et al.,
2019) for quantifying the stability of \emph{g} genotypes
conducted in \emph{e} environments using linear mixed-effect models.
}
\details{
According to Olivoto et al. (2019a), WAASB(The weighted average of
absolute scores) is computed considering all Interaction Principal
Component Axis (IPCA) from the Singular Value Decomposition (SVD)
of the matrix of genotype-environment interaction (GEI) effects
generated by a linear mixed-effect model, as follows:

\loadmathjax
\mjsdeqn{
WAASB_i = \sum_{k = 1}^{p} |IPCA_{ik} \times EP_k|/
\sum_{k = 1}^{p}EP_k}

where \mjseqn{WAASB_i} is the weighted average of absolute scores
of the \emph{i}th genotype; \mjseqn{IPCA_{ik}} is the score of the \emph{i}th
genotype in the \emph{k}th Interaction Principal Component Axis (IPCA);
and \mjseqn{EP_k} is the explained variance of the \emph{k}th IPCA for
\emph{k = 1,2,..,p}, considering \mjseqn{p=min(g-1; e-1)}.

Further, \mjseqn{WAASBY_i} is a superiority or simultaneous
selection index allowing weighting between mean performance
and stability
\mjsdeqn{
WAASBY_i=\frac{\left({rY}_i\times\theta_Y\right)+
\left({rW}_i\times\theta_s\right)}{\theta_Y+\theta_s}
}
, where \mjseqn{WAASBY_i} is the superiority index for genotype
\mjseqn{\it{i}} that weights between mean performance and stability;
\mjseqn{\theta_Y} and \mjseqn{\theta_s} are the weights for
mean performance and stability, respectively; \mjseqn{{rY_i}} and
\mjseqn{{rW}_i} are the rescaled values for mean performance
\mjseqn{\bar{Y_i}} and stability \mjseqn{W_i}, respectively of
the genotype \emph{i}. For the details of calculations, rescaling
and mathematics notations see (Olivoto et al., 2019).

Finally, \mjseqn{rYWAASB_i} index is the sum of the ranks
(or in fact the rank of sum of ranks of  the trait and
WAASB index) as follows:
(\mjseqn{rY_i}) and WAASB index (\mjseqn{rWAASB_i}) for each
individual:

\mjsdeqn{
rYWAASB_i = {rY_i} + {rWAASB_i}} or: =
\mjsdeqn{rank{{rY_i} + {rWAASB_i}}}.

The input format of table of data(NA free), here \emph{maize} data,
should be as follows:

\tabular{rrrr}{
\strong{GEN} \tab \strong{Y} \tab \strong{WAASB} \tab \strong{WAASBY}\cr
Dracma \tab 262.22 \tab 0.81 \tab 81.6\cr
DKC6630 \tab 284.04 \tab 2.20 \tab 88.5\cr
NS770 \tab 243.48 \tab 0.33 \tab 71.4\cr
...
}
}
\examples{
# Case 1:  Higher trait values are preferred. For instance grain yield
# in cereals is a trait which its higher values are preferred and ranking
# is performed from the higher to lower values i.e. 1st, 2nd, 3rd etc
# in maize dataset.
\donttest{
data(maize)
ranki(maize) # or: ranki(maize, lowt = FALSE)
}
# Case 2:  In this case, the lower values of the given trait are preferred.
# For instance days to maturity (dm) and plant height are traits where their
# lower values are preferred.
\donttest{
data(dm)
ranki(dm, lowt = TRUE)
}
}
\references{
Olivoto, T., Lúcio, A., DC, da Silva, J.A.G., Sari, B.G.
and Diel, M. 2019. Mean performance and stability in
multi-environment trials II: Selection based on multiple
traits. Agronomy Journal, 111(6):2961-2969.

Olivoto, T., & Lúcio, A.D.C.2020. metan: An R package for
multi‐environment trial analysis. Methods in Ecology and
Evolution, 11(6), 783-789.

Kang, M.S. 1988. “A Rank-Sum Method for Selecting High-Yielding,
Stable Corn Genotypes.” Cereal Research Communications 16: 113–15.
}
\author{
{
Ali Arminian \href{mailto:abeyran@gmail.com}{abeyran@gmail.com}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nbclust.R
\name{nbclust}
\alias{nbclust}
\title{Data read and estimate the cluster number}
\usage{
nbclust(datap, verbose = FALSE)
}
\arguments{
\item{datap}{The data set}

\item{verbose}{If \code{verbose = TRUE} then some results are}
}
\value{
Returns a data frame
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{nbclust()} reads and prepares the data, and
determine the optimum number of clusters using Average
Silhouette Method by \code{factoextra} package.
The average silhouette approach assesses the quality of
clustering by evaluating how well each object fits within
its cluster. A high average silhouette width signifies
effective clustering. This method calculates the average
silhouette for different values of k, and the optimal
number of clusters (k) is the one that maximizes the
average silhouette across a range of potential k values.
}
\details{
The \code{silhouette} coefficient (SC) refers to a criterion
to decide number of clusters.It is defined as follows.
Though there are numerous methods determining number
of clusters such as \verb{the gap statistic} etc.

\loadmathjax
\mjsdeqn{SC = maxK{\bar{S}K}}.

In other words, for each observation \mjseqn{i}, the
\verb{silhouette width} \mjseqn{s(i)} is defined as follows:
Put \mjseqn{a(i)} = average dissimilarity between i and all
other points of the cluster to which i belongs (if i is the only
observation in its cluster, \mjseqn{s(i):= 0} without further
calculations). For all other clusters C, put \mjseqn{d(i, C)}
= average dissimilarity of i to all observations of C.
The smallest of these \mjseqn{d(i, C)} is
\mjseqn{b(i)=min\it(C) d(i, C)}, and can be seen as the
dissimilarity between i and its “neighbor” cluster, i.e.,
the nearest one to which it does not belong. Finally,
\mjsdeqn{s(i)=\frac{b(i)-a(i)}{max(a(i), b(i))}}
\itemize{
\item Note: The clustering methods can be: "average", "centroid",
"complete", "mcquitty", "median", "single", "ward.D", "ward.D2"
and, Distance methods can be as: "binary", "canberra", "euclidean",
"manhattan", "minkowski", "maximum", "pearson", "spearman", "kendall"
which may be used in \code{shipunov} or \code{factoextra} packages.
In this package we just applied \code{average=UPGMA} and \code{ward.D} algorithms.
}
}
\examples{
\donttest{
library(factoextra)
data(maize)
maize <- as.data.frame(maize)
row.names(maize) <- maize[, 1]
maize[, 1] = NULL
GEN <- row.names(maize)
maize <- scale(maize)
nbclust(maize, verbose = FALSE)

# Performing bootstrap or jackknife clustering
# by shipunov package. The examples should be run in the
# console manually due to problems occurs in the ORPHANED
# package `shipunov`.
#
# library(shipunov)
# 1- Bootstrap clustering:
# data.jb <- Jclust(maize,
#   method.d = "euclidean",
#    method.c = "average", n.cl = 2,
#     bootstrap = TRUE)
#
# plot.Jclust(data.jb, top=TRUE, lab.pos=1,
#  lab.offset=1, lab.col=2, lab.font=2)
# Fence(data.jb$hclust, GEN)
#
# data.jb <- Jclust(maize,
#  method.d = "euclidean",
#    method.c = "ward.D", n.cl = 2,
#     bootstrap = TRUE)
#
# plot.Jclust(data.jb, top=TRUE, lab.pos=1,
#            lab.offset=1, lab.col=2, lab.font=2)
# Fence(data.jb$hclust, GEN)
#
# if(verbose = TRUE):
# cat("\nnumber of iterations:\n", data.jb$iter, "\n")
#
# for "bootstrap":
# data.jb$mat <- as.matrix((data.jb$mat))
# data.jb$mat
# cat("\nmatrix of results:\n", data.jb$mat, "\n")
# cat("clustering info, by eucledean distance measure:\n")
# print(data.jb$hclust)
# cat("groups:\n", data.jb$gr, "\n")
# cat("\nsupport values:\n", data.jb$supp, "\n")
# cat("\nnumber of clusters used:\n", data.jb$n.cl, "\n")

# 2- Jackknife clustering:
# data.jb <- Bclust(maize,
#   method.d = "euclidean", method.c = "average",
#    bootstrap = FALSE)
# plot(data.jb)
#
# data.jb <- Bclust(maize,
#   method.d = "euclidean", method.c = "ward.D",
#    bootstrap = FALSE)
# plot(data.jb)
#
# if(verbose = TRUE):
# For"jackknife":
# cat("Consensus:\n", data.jb$consensus, "\n")
# cat("Vlaues:\n", data.jb$values, "\n")
}
}
\references{
Lletı, R., Ortiz, M.C., Sarabia, L.A., Sánchez, M.S. 2004.
Selecting variables for k-means cluster analysis by using
a genetic algorithm that optimizes the silhouettes,
Analytica Chimica Acta, 515(1): 87-100.

Rousseeuw, P.J. (1987) Silhouettes: A graphical aid to the
interpretation and validation of cluster analysis.
J. Comput. Appl. Math., 20, 53-65.

https://uc-r.github.io/
}
\author{
{
Ali Arminian \href{mailto:abeyran@gmail.com}{abeyran@gmail.com}
}
}
